#pragma NOIV	// tells compiler not to generate interrupt vectors
				// Note that we have to edit USBJmpTB if we wanna add
				// other interrupts later

/*=============================================================================
	File:      SyncBulk.c
	Contents:   Hooks required to implement USB peripheral function.
	Comments:	Original file was given by a Cypress example, this
				one is edited only so there might me reserved rights:

				Copyright (c) 2000 Cypress Semiconductor All rights reserved
=============================================================================*/

#include "fx2.h"
#include "fx2regs.h"			// includes FX2 register adressing
#include "fx2sdly.h"            // includes SYNCDELAY macro

/*=============================================================================
	Pre-Processor defines:   	
=============================================================================*/
// nothing to do here 

/*=============================================================================
	Global variables:
	External variables which belong to the Anchor USB Framework and are found
	in FW.c   	
=============================================================================*/

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;			   	// see TRM Chapter 6 Power Management
extern BOOL Rwuen;				// see TRM Chapter 6 Power Management
extern BOOL Selfpwr;			// see TRM Chapter 6 Power Management

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings


// FX2 USB PortA = slave fifo enable(s) when IFCFG[1:0] = 11
//sbit PA0 = IOA ^ 0; // alt. function INT 0
//sbit PA1 = IOA ^ 1; // alt. function INT 1
//sbit PA2 = IOA ^ 2; // is sloe
//sbit PA3 = IOA ^ 3; // alt. function WU2
//sbit PA4 = IOA ^ 4;	// is FIFO Adress 0
//sbit PA5 = IOA ^ 5; // is FIFO Adress 1
//sbit PA6 = IOA ^ 6; // is PKTEND
//sbit PA7 = IOA ^ 7; // is FLAGS / SLCS


void SetupCommand(void);

/*=============================================================================
	Task Dispatcher hooks
	The following hooks are called by the task dispatcher.
=============================================================================*/

void TD_Init(void)             // Called once at startup
{
	/*=============================================================================
 		Just commented some defaults of the original file, 
		maybe the are usefull or deleted later ...
 	
 	  	// set the CPU clock to 48MHz
		CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1) ;
		CPUCS |= bmCLKOE;
	=============================================================================*/
	//some FX2 CPU settings:
	CPUCS = 0x12;

	// These registers require a synchronization delay, see section 15.14 (P15-105)
  	// FIFORESET	FIFOPINPOLAR	INPKTEND
	// EPxBCH:L		EPxFIFOPFH_L	EPxAUTOINLENH:L
	// EPxFIFOCFG	EPxGPIFFLGSEL	PINFLAGSAB
	// PINFLAGSCD	EPxFIFOIE		EPxFIFOIRQ
	// GPIFIE		GPIFIRQ			UDMACRCH:L
	// GPIFADRH:L	GPIFTRIG		EPxGPIFTRIG
	// OUTPKTEND	REVCTL			GPIFTCB3
	// GPIFTCB2		GPIFTCB1		GPIFTCB0

	/*=======================================================================
	  Now we configure IFCLK Output and slave FIFO Mode:
	  - external clock: b7 = 1 => uses internal Clock
						b6 = 0 => IFCLK Speed : 30MHz
						b5 = 1 => enable Clock Output
	  					b4 = 0 => do not invert IFCLK
						b3 = 0 => synchronous transmission
						b2 = 0 => no effect, only GPIF related
						b1 and b0 = 11 => activate slave FIFO mode
	=======================================================================*/

	IFCONFIG = 0xA3; // 1010.0011

	/*======================================================================
	  Now the Endpoints will be configured. Up here only the basics are set
	  configuration of EP FIFOs will be done later.
	  default: all endpoints have their VALID bit set
  	  default: TYPE1 = 1 and TYPE0 = 0 --> BULK  
  	  default: EP2 and EP4 DIR bits are 0 (OUT direction)
  	  default: EP6 and EP8 DIR bits are 1 (IN direction)
  	  default: EP2, EP4, EP6, and EP8 are double buffered

      we are not using the default values, so the following is necessary:
	  deactive EP1IN/OUT, EP2, EP4, EP8
	  configure EP6 as ISO IN quad buffered at 8bit WORDWIDE, 1024byte
	  All the EP Configurations need SYNCDELAYs; See TRM section 15.14
	=======================================================================*/
	// Configuration of EP1:  
	EP1OUTCFG = 0x00; SYNCDELAY;	// deactivate endpoint 1 OUT
  	EP1INCFG = 0x00; SYNCDELAY;		// deactivate endpoint 1 IN
	
	// Configuration of EP2
	EP2CFG = 0x00; SYNCDELAY;		// deactivate EP2

	// Configuration of EP4          
  	EP4CFG = 0x00; SYNCDELAY;		// deactivate endpoint 4

	/*==================================================================================
	  Now we configure EP6CFG:
	  b7 = 1 => VALID: activate EP = 1 / deactive EP = 0
	  b6 = 1 => DIR: OUT = 0 / IN = 1
	  [b5:b4] = 10 => Type : 00 = invalid / 01 = ISOChron / 10 = BULK / 11 = Interrupt  
	  b3 = 1 => SIZE : 0 = 512 / 1 = 1024
	  b2 = 0 => no effect
	  [b1:b0] = 00 => buffer Type: 00 = Quad / 01 = invalid / 10 = double / 11 = tripple
	===================================================================================*/
    EP6CFG = 0xE8; SYNCDELAY;		// 1101.1000

  	REVCTL = 0x03;	// MUST set REVCTL.0 and REVCTL.1 to to '1'
	SYNCDELAY;
	FIFORESET = 0x80; // activate NAK all to avoid race conditions
	SYNCDELAY;
	FIFORESET = 0x02; // reset fifo buffer of EP2
	SYNCDELAY;
	FIFORESET = 0x04; // reset fifo buffer of EP4
	SYNCDELAY;
	FIFORESET = 0x06; // reset fifo buffer of EP6
	SYNCDELAY;
	FIFORESET = 0x08; // reset fifo buffer of EP8
	SYNCDELAY;
	FIFORESET = 0x00; // deactivate NAK-ALL, normal operation mode 


	/*==================================================================================
	  Configuration of deactivated EPs, maybe the following is completly useless
	  B7,B5,B4,B1 no effect (no connection)
	  B6 In Full Minus 1
	  B3 AUTO IN
      B2 enables ZERO Length Packet
	  B0 Word Width 0 = 8bit / 1 = 16bit
	==================================================================================*/
    EP2FIFOCFG = 0x40; SYNCDELAY;  //0100.0000
	EP4FIFOCFG = 0x40; SYNCDELAY;  //0100.0000
 	EP8FIFOCFG = 0x40; SYNCDELAY;  //0100.0000
     
	/*==================================================================================
	  Now we configure EP6FIFOCFG:
	  b7 = 0 => no effect
	  b6 = 1 => IN Full Minus 1, full flag comes up 1 bit earlier
	  [b5:b4] = 00 => no effect
	  b3 = 1 => AUTOIN
	  b2 = 1 => ZERO Length paket if pktend is set and fifo is empty
	  b1 = 0 => no effect
	  b0 = 0 => Wordwide 0 = 8bit / 1 = 16bit 
	===================================================================================*/
	EP6FIFOCFG = 0x4C; SYNCDELAY; // 0100.1100
	
	/*==================================================================================
	  Now we configure PINFLAGSAB:
	 [b7:b4] => Flag B related
	 [b3:b0] => Flag A related
	 0110 => EP6 programmable Full Flag
	 1010 => EP6 Empty Flag
	 1110 => EP6 Full Flag
	 1110.1010 => Flag B = EP6 Full Flag / Flag A = EP6 Empty Flag
	 REMEMBER! The PINFLAGS are arranged BACKWARDS!
	 PINFLAGSAB = [FLAGB ; FLAGA] / PINFLAGSCD = [FLAGD ; FLAGC]
	===================================================================================*/
	PINFLAGSAB = 0xEA;	// 1110.1010
	SYNCDELAY;

	/*===================================================================================
	  Configuration of Pin polarity, default all signaling pins are active low!
	  We are not going to change this, so the following could also be skipped
    ===================================================================================*/
	FIFOPINPOLAR = 0x00; // as it is set in defaults
	SYNCDELAY;			
	
	/*==================================================================================
	  Now we configure length to AUTO IN
	 EP6AUTOINLENH :
	 	[b7:b3] = 0 => no effect
		[b2:b0] = 010 => High bits of Paket length => 010 = 512 => 0x02
	 EP6AUTOINLENL :
		[b7:b0] = 0000.0000 => low bits of paket length = 512 => 0x00	
	===================================================================================*/
	EP6AUTOINLENH = 0x02; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x00; //	


	/*==================================================================================
	  Now we configure PORT A CFG
	  Use Port A as standart Port Pins
	===================================================================================*/
	PORTACFG = 0x00;
	SYNCDELAY;
}


void TD_Poll(void)              // Called repeatedly while the device is idle
{
}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   
// Called when a Set Configuration command is received
{
   if (EZUSB_HIGHSPEED())
	{
	// ... FX2 in highspeed mode
	EP6AUTOINLENH = 0x02; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x00; //	to 512 bytes
	}
	else
	{
	// ... FX2 in fullspeed mode
	EP6AUTOINLENH = 0x00; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x40; //	to 64 bytes
	}
	Configuration = SETUPDAT[ 2 ];
	return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   
// Called when a Get Configuration command is received
{
   EP0BUF[ 0 ] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       
// Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[ 2 ];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       
// Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
  return(TRUE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
   SetupCommand();	// speed up, if it is 'here' -> see USB Spec.
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   /* Commented defaults:
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
   */

   if ( EZUSB_HIGHSPEED() )
   { 
   	pConfigDscr = pHighSpeedConfigDscr;
	pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
	pConfigDscr = pFullSpeedConfigDscr;
	pOtherConfigDscr = pHighSpeedConfigDscr;
   }
   
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ	    	
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      //((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
     // ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }
   else
   {
	pConfigDscr = pFullSpeedConfigDscr;
	pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}
/*
void ISR_int1(void) interrupt 2 using 2
{
}
*/